%ifndef _SLOXX_INC_
%define _SLOXX_INC_

; -=( MemCpy )=-------------------------------------------------------------
%macro MemCpy 2-3+

push es
push ds
pop es
%ifid %2
mov si,%2
mov cx,%3
%else
%ifdef DATASEG
segment _DATA
%else
%define DATASEG
segment _DATA CLASS=DATA
%endif
%%Dat	db %2
%if %0 = 3
	db %3
%endif
%%DatEnd:
segment _TEXT
mov si,%%Dat
mov cx,%%DatEnd - %%Dat
%endif
mov di,%1
rep movsb
pop es
%endmacro

; -=( tileIdx )=------------------------------------------------------------
; -=( Berechnet: MAP_WIDTH*Y + X )=-
; -=( IN: %1 - X (reg,nicht ax), %2 - Y (reg,nicht ax); OUT: bx - index )=-
; --------------------------------------------------------------------------
%macro tileIdx 2
        push ax
        push dx
        
        mov ax,MAP_WIDTH                ;ax = maxX*Y + X
        mul %2
        pop dx

        add ax,%1
        mov bx,ax                       ;bx = index
        
        pop ax
%endmacro

; -=( Struc obj )=----------------------------------------------------------
struc obj		;Bewegtes sprite
  objType	resw 1  ;Typ (0 - Player, 1..? - Monster)

;P/64 bedeutet Pixel/64
;64 P/64 entspricht 1 Pixel, 32 P/64 entspricht 1/2 Pixel, ...
;Wird beim Anzeigen abgerundet (Pixel = P/64 >> 6)

  objX		resw 1	;X-Koordinate auf Map in P/64, unsigned
  objY		resw 1	;Y-    "       "   "   "   " ,     "

  objSpeedX	resw 1	;Laufgeschwindigkeit    in P/64, unsigned
  objSpeedJump	resw 1	;Springgeschwindigkeit   "   "	       "
  objSpeedFall  resw 1  ;Fallgeschwindigkeit     "   "         "
  objGravity	resw 1  ;Anziehungskraft	 "   "         "

  objOnTile	resb 1  ;Steht obj auf festem Boden?
  objOnLadder	resb 1	;  "    "   "  Leiter?
  objOnSwitch   resb 1  ;  "    "   "  Schalter?
  objTrapped	resb 1	;Ist obj in einem ForceField gefangen?
  
  objDummy	resb 1  ;Dummy um Alignment zu behalten

  objAdvCount	resb 1  ;Wie oft advanced seit AdvCount auf 0 gesetzt wurde?

  objFace	resw 1	;Aktives Face,  siehe Sprite
  objFrame	resw 1  ;Aktives Frames,  "      "

  objSpritePtr  resw 1  ;Zeiger auf einen Sprite-Struc
endstruc

%macro Obj 4
	istruc obj
	at objX, 	dw 0
	at objY, 	dw 0
	at objSpeedX, 	dw %1
	at objSpeedJump,dw %2
	at objSpeedFall,dw %3
	at objGravity,	dw 0
	at objOnTile,	db 0
	at objOnLadder,	db 0
	at objOnSwitch, db 0
	at objTrapped,	db 0
	at objDummy,	db 0
	at objAdvCount,	db 0
	at objFace,	dw 0
	at objFrame,	dw 0
	at objSpritePtr,dw %4
	iend
%endmacro

TILE_WIDTH      equ 12 	;Pixel
TILE_HEIGHT     equ 12 	;Pixel
MAP_WIDTH       equ 26 	;Tiles
MAP_HEIGHT      equ 15 	;Tiles

MAP_SIZE        equ MAP_WIDTH * MAP_HEIGHT

SCREEN_W	equ 360	;Pixel
SCREEN_H	equ 200 ;Pixel

MAX_MONSTERS    equ 10

TILBG	equ 0
TILBLK	equ 1
TILCOIN	equ 2
TILEXIT	equ 3
TILLADR	equ 4
TILAUTL	equ 5
TILAUTR	equ 6
TILFFR	equ 7
TILFFG  equ 8
TILFFB	equ 9
TILFFY	equ 10
TILSWR	equ 11
TILSWG	equ 12
TILSWB	equ 13
TILSWY	equ 14
TILSLIM	equ 15
TILGEM	equ 16
TILPAC	equ 17
TILBALL	equ 18
TIL1UP	equ 19
TILMAN	equ 20
TILAPPR	equ 21
TILDSAP equ 22
TILLETR equ 23
TILULTR	equ 24
TILLAST equ 24

; -=( Struc spr )=----------------------------------------------------------
struc spr               ;Sprite
  sprNoOfFaces  resw 1  ;Wieviele Faces enthlt das Sprite?
  sprNoOfFrames resw 1  ;    "   Frames    "    jedes Face?

  sprFaceSize   resw 1  ;Gre eines Faces in Bytes (NoOfFrames * FrameSize)
  sprFrameSize  resw 1  ;  "     "  Frames  "   "

  sprAdvDelay   resb 1  ;Nur advancen, wenn objAdvCount == sprAdvDelay

  sprData:              ;SpriteData folgt nach dem Struct
endstruc                ;(FaceSize * NoOfFaces bytes)

%macro Spr 3            ;Parameter: NoOfFaces, NoOfFrames, AdvDelay
	istruc spr
  at sprNoOfFaces,      dw %1
  at sprNoOfFrames,     dw %2
  at sprFaceSize,       dw 0
  at sprFrameSize,      dw 0
  at sprAdvDelay,       db %3
  at sprData
	iend
%endmacro

%endif
